/*  File: analyser-ZQ.cc
 *  Project: Deductor
 *  Created by Zoltán Nagy on 9/8/13.
 *  Modified by D. Soper on 3 November 2017.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
/*   local headers  */
#include "analyzer-ZQ.h"

void UserZQ::initfunc()
{
  /*   Create object to calculate the total cross section   */
  Hist0D::phys(1, "Total cross section");
  
  /*   Create a histogram to calculate the Q-distribution   */
  unsigned int nbins = 67;
  double Qmin = 350.0_GeV, Qmax = 7050.0_GeV;
  auto bins = spacing<hist1d>::linear(nbins, Qmin, Qmax);
  
  Hist1D::phys(1, "$Z/\\gamma$ $Q$ distribution - Full", bins);
  Hist1D::phys(2, "$Z/\\gamma$ $Q$ distribution - NoDelta", bins);
  Hist1D::phys(3, "$Z/\\gamma$ $Q$ distribution - ThresholdMain", bins);
  Hist1D::phys(4, "$Z/\\gamma$ $Q$ distribution - Std", bins);
  Hist1D::phys(5, "$Z/\\gamma$ $Q$ distribution - LO", bins);
  count = 0;
}

void UserZQ::userfunc(const shower_history<hhc>& history)
{
  constexpr double tonb = 389379.338;
  bool newHard = true;
  /*   the shower stage after the shower evolution   */
  for(auto& hard : history.hard())
  {
    if(newHard)  // Count the hard event once only.
    {
      newHard = false;
  
      double wHard = hard.weights.unitary.main*color_weight(hard)*tonb;
      /*    hard events   */
      auto& q = hard.state;
      /*   mass of the ee pair   */
      auto pee = q[-2].momentum + q[-3].momentum;
      auto eemass = pee.mag();
      if(eemass > 350.0 && eemass < 7050.0)
      {
        Hist1D::physfill(5, wHard, dirac{}, eemass);
      }
    }
  }
  // Done with hard event.
  // Cross section after showering.
  for(auto& showered : history)
  {
    /*   Calculate weights at the end of the shower   */
    // The weight for a probability conserving shower
    double wStd = showered.weights.unitary.main*color_weight(showered)*tonb;
    // The main threshold factor, omitting P^{reg} term and the Delta term.
    double wThresholdMain = wStd*showered.weights.threshold.main;
    // Now include the P^{reg} term.
    double wNoDelta = wThresholdMain*showered.weights.threshold.reg;
    // Now include the Delta term also, giving the full result.
    double wFull = wNoDelta*showered.weights.threshold.delta;
    /*   Total cross section  */
    Hist0D::physfill(1, wFull);
    /*   showered events   */
    auto& p = showered.state;
    /*   mass of the ee pair   */
    auto pee = p[-2].momentum + p[-3].momentum;
    auto eemass = pee.mag();
    
    /*   Z/gamma-mass distribution   */
    if(eemass > 350.0 && eemass < 7050.0)
    {
      Hist1D::physfill(1, wFull, dirac{}, eemass);
      Hist1D::physfill(2, wNoDelta, dirac{}, eemass);
      Hist1D::physfill(3, wThresholdMain, dirac{}, eemass);
      Hist1D::physfill(4, wStd, dirac{}, eemass);
    }
  }
}
