/*  File: mod-jets.cc
 *  Project: Deductor
 *  Created by Zoltán Nagy on 9/8/13.
 *  Modified by D. Soper on 3 August 2017.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*   Deductor headers  */
#include <deductor.h>
#include <proc-hhcjets.h>


/*   Analyser headers  */
#include "analyzer-jets.h"


auto my_jet_module(const deductor_input&)
{
  /*  Deductor object  */
  deductor<hhc> kernel{qcd::ct14n};
  
  /*    Hard process     */
  auto hard = [=](const model_ref& mdl) -> hhcjets
  {
    int proton = 2212;
    double Ecm = 13.0_TeV;
    double scalefacpert = 0.707;
    double scalefacshowerstart = 1.50;
    auto pTlist = {
      100.0, 300.0, 400.0, 600.0, 800.0, 1200.0, 1600.0,
      2000.0, 2500.0, 3000.0, 3500.0, 4000.0, 4500.0, 5000.0
    };
    
    return {mdl, Ecm, pTlist, scalefacpert, scalefacshowerstart, proton, proton};
  };
  
  
  /*   Jet cross section pT distribution */
  shower_engine_opts opts;
  opts.max_color_suppression = 4u;
  opts.allow_color_branching = false;
  
  /* kernel with three UserJets instances */
  kernel("JetCalculations", opts, hard, {
    new UserJets{"Jets-R=0.2", {fj::antikt_algorithm, 0.2}},
    new UserJets{"Jets-R=0.4", {fj::antikt_algorithm, 0.4}},
    new UserJets{"Jets-R=0.7", {fj::antikt_algorithm, 0.7}},
  });
}

/*   Always export your module functions!   */
export_my_module jets_module = {
  {"Jets-13TeV", my_jet_module}
};





